@@ -9,61 +9,58 @@ Based on research by Marco Lui and Tim Baldwin.
 See LICENSE file for more info.
 """
 
-import argparse
-import base64
 import bz2
 import json
 import logging
 import lzma
+import pickle
 
+from base64 import b64decode
 from collections import Counter
 from pathlib import Path
 from urllib.parse import parse_qs
 
-from wsgiref.simple_server import make_server
-from wsgiref.util import shift_path_info
-
 import numpy as np
 
-# cpickle on Python3
-try:
-    import _pickle as pickle
-# default
-except ImportError:
-    import pickle
 
+LOGGER = logging.getLogger(__name__)
 
-logger = logging.getLogger(__name__)
-
-# Convenience methods defined below will initialize this when first called.
+# model defaults
 IDENTIFIER = None
 MODEL_FILE = 'data/model.plzma'
-
-# Defaults for inbuilt server
-HOST = None  # leave as none for auto-detect
-PORT = 9008
-FORCE_WSGIREF = False
-
 NORM_PROBS = False  # Normalize output probabilities.
 # NORM_PROBS defaults to False for a small speed increase. It does not
 # affect the relative ordering of the predicted classes. It can be
 # re-enabled at runtime - see the readme.
 
 
+def load_model(path=None):
+    """
+    Convenience method to set the global identifier using a model at a
+    specified path.
+
+    @param path to model
+    """
+    LOGGER.debug('initializing identifier')
+    global IDENTIFIER
+    if path is None:
+        IDENTIFIER = LanguageIdentifier.from_pickled_model(MODEL_FILE)
+    else:
+        IDENTIFIER = LanguageIdentifier.from_modelpath(path)
+
+
 def set_languages(langs=None):
     """
     Set the language set used by the global identifier.
 
     @param langs a list of language codes
     """
-    global IDENTIFIER
     if IDENTIFIER is None:
-        IDENTIFIER = load_model()
-
+        load_model()
     return IDENTIFIER.set_languages(langs)
 
 
-def classify(instance):
+def classify(instance, datatype='uint16'):
     """
     Convenience method using a global identifier instance with the default
     model included in langid.py. Identifies the language that a string is
@@ -72,11 +69,9 @@ def classify(instance):
     @param instance a text string. Unicode strings will automatically be utf8-encoded
     @returns a tuple of the most likely language and the confidence score
     """
-    global IDENTIFIER
     if IDENTIFIER is None:
-        IDENTIFIER = load_model()
-
-    return IDENTIFIER.classify(instance)
+        load_model()
+    return IDENTIFIER.classify(instance, datatype=datatype)
 
 
 def rank(instance):
@@ -88,10 +83,8 @@ def rank(instance):
     @param instance a text string. Unicode strings will automatically be utf8-encoded
     @returns a list of tuples language and the confidence score, in descending order
     """
-    global IDENTIFIER
     if IDENTIFIER is None:
-        IDENTIFIER = load_model()
-
+        load_model()
     return IDENTIFIER.rank(instance)
 
 
@@ -104,10 +97,8 @@ def cl_path(path):
     @param path path to file
     @returns a tuple of the most likely language and the confidence score
     """
-    global IDENTIFIER
     if IDENTIFIER is None:
-        IDENTIFIER = load_model()
-
+        load_model()
     return IDENTIFIER.cl_path(path)
 
 
@@ -120,30 +111,17 @@ def rank_path(path):
     @param path path to file
     @returns a list of tuples language and the confidence score, in descending order
     """
-    global IDENTIFIER
     if IDENTIFIER is None:
-        IDENTIFIER = load_model()
-
+        load_model()
     return IDENTIFIER.rank_path(path)
 
 
-def load_model(path=None):
-    """
-    Convenience method to set the global identifier using a model at a
-    specified path.
-
-    @param path to model
-    """
-    logger.debug('initializing identifier')
-    if path is None:
-        return LanguageIdentifier.from_pickled_model(MODEL_FILE)
-    return LanguageIdentifier.from_modelpath(path)
-
-
 class LanguageIdentifier:
     """
     This class implements the actual language identifier.
     """
+    __slots__ = ['nb_ptc', 'nb_pc', 'nb_numfeats', 'nb_classes', 'tk_nextmove', 'tk_output',
+                 'norm_probs', '__full_model']
 
     # new version: speed-up
     @classmethod
@@ -152,7 +130,7 @@ class LanguageIdentifier:
         filepath = str(Path(__file__).parent / pickled_file)
         with lzma.open(filepath) as filehandle:
             nb_ptc, nb_pc, nb_classes, tk_nextmove, tk_output = pickle.load(filehandle)
-        nb_numfeats = int(len(nb_ptc) / len(nb_pc))
+        nb_numfeats = len(nb_ptc) // len(nb_pc)
 
         # reconstruct pc and ptc
         nb_pc = np.array(nb_pc)
@@ -164,8 +142,8 @@ class LanguageIdentifier:
     @classmethod
     def from_modelstring(cls, string, *args, **kwargs):
         # load data
-        nb_ptc, nb_pc, nb_classes, tk_nextmove, tk_output = pickle.loads(bz2.decompress(base64.b64decode(string)))
-        nb_numfeats = int(len(nb_ptc) / len(nb_pc))
+        nb_ptc, nb_pc, nb_classes, tk_nextmove, tk_output = pickle.loads(bz2.decompress(b64decode(string)))
+        nb_numfeats = len(nb_ptc) // len(nb_pc)
 
         # reconstruct pc and ptc
         nb_pc = np.array(nb_pc)
@@ -201,8 +179,8 @@ class LanguageIdentifier:
                 # Windows this causes a RuntimeWarning, so we explicitly
                 # suppress it.
                 with np.errstate(over='ignore'):
-                    # old: pd = (1/np.exp(pd[None,:] - pd[:,None]).sum(1))
-                    pd = (pd - np.min(pd)) / np.ptp(pd)
+                    # legacy formula, there are possibly better alternatives
+                    pd = 1/np.exp(pd[None,:] - pd[:,None]).sum(1)
             return pd
 
         self.norm_probs = apply_norm_probs
@@ -212,7 +190,7 @@ class LanguageIdentifier:
         self.__full_model = nb_ptc, nb_pc, nb_classes
 
     def set_languages(self, langs=None):
-        logger.debug("restricting languages to: %s", langs)
+        LOGGER.debug("restricting languages to: %s", langs)
 
         # Unpack the full original model. This is needed in case the language set
         # has been previously trimmed, and the new set is not a subset of the current
@@ -229,7 +207,7 @@ class LanguageIdentifier:
             # to speed up processing.
             for lang in langs:
                 if lang not in nb_classes:
-                    raise ValueError("Unknown language code %s" % lang)
+                    raise ValueError(f"Unknown language code {lang}")
 
             subset_mask = np.fromiter((l in langs for l in nb_classes), dtype=bool)
             self.nb_classes = [c for c in nb_classes if c in langs]
@@ -265,15 +243,15 @@ class LanguageIdentifier:
 
     def nb_classprobs(self, fv):
         # compute the partial log-probability of the document given each class
-        pdc = np.dot(fv, self.nb_ptc)
+        pdc = np.dot(fv, self.nb_ptc)  # fv @ self.nb_ptc
         # compute the partial log-probability of the document in each class
         return pdc + self.nb_pc
 
-    def classify(self, text):
+    def classify(self, text, datatype='uint16'):
         """
         Classify an instance.
         """
-        fv = self.instance2fv(text)
+        fv = self.instance2fv(text, datatype=datatype)
         probs = self.norm_probs(self.nb_classprobs(fv))
         cl = np.argmax(probs)
         return self.nb_classes[cl], probs[cl]
@@ -303,78 +281,11 @@ class LanguageIdentifier:
         return path, retval
 
 
-# Based on http://www.ubacoda.com/index.php?p=8
-query_form = """
-<html>
-  <head>
-    <meta http-equiv="Content-type" content="text/html; charset=utf-8">
-    <title>Language Identifier</title>
-    <script src="//ajax.googleapis.com/ajax/libs/jquery/1.7.2/jquery.min.js" type="text/javascript"></script>
-    <script type="text/javascript" charset="utf-8">
-      $(document).ready(function() {{
-        $("#typerArea").keyup(displayType);
-
-        function displayType(){{
-          var contents = $("#typerArea").val();
-          if (contents.length != 0) {{
-            $.post(
-              "/rank",
-              {{q:contents}},
-              function(data){{
-                for(i=0;i<5;i++) {{
-                  $("#lang"+i).html(data.responseData[i][0]);
-                  $("#conf"+i).html(data.responseData[i][1]);
-                }}
-                $("#rankTable").show();
-              }},
-              "json"
-            );
-          }}
-          else {{
-            $("#rankTable").hide();
-          }}
-        }}
-        $("#manualSubmit").remove();
-        $("#rankTable").hide();
-      }});
-    </script>
-  </head>
-  <body>
-    <form method=post>
-      <center><table>
-        <tr>
-          <td>
-            <textarea name="q" id="typerArea" cols=40 rows=6></textarea></br>
-          </td>
-        </tr>
-        <tr>
-          <td>
-            <table id="rankTable">
-              <tr>
-                <td id="lang0">
-                  <p>Unable to load jQuery, live update disabled.</p>
-                </td><td id="conf0"/>
-              </tr>
-              <tr><td id="lang1"/><td id="conf1"></tr>
-              <tr><td id="lang2"/><td id="conf2"></tr>
-              <tr><td id="lang3"/><td id="conf3"></tr>
-              <tr><td id="lang4"/><td id="conf4"></tr>
-            </table>
-            <input type=submit id="manualSubmit" value="submit">
-          </td>
-        </tr>
-      </table></center>
-    </form>
-
-  </body>
-</html>
-"""
-
-
 def application(environ, start_response):
     """
     WSGI-compatible langid web service.
     """
+    from wsgiref.util import shift_path_info
     try:
         path = shift_path_info(environ)
     except IndexError:
@@ -409,29 +320,24 @@ def application(environ, start_response):
             # Unsupported method
             status = '405 Method Not Allowed'  # HTTP Status
             response = {
-              'responseData': None,
-              'responseStatus': 405,
-              'responseDetails': '%s not allowed' % environ['REQUEST_METHOD']
+                'responseData': None,
+                'responseStatus': 405,
+                'responseDetails': f"{environ['REQUEST_METHOD']} not allowed",
             }
 
         if data is not None:
             if path == 'detect':
                 pred, conf = classify(data)
-                responseData = {'language': pred, 'confidence': conf}
+                response_data = {'language': pred, 'confidence': conf}
             elif path == 'rank':
-                responseData = rank(data)
+                response_data = rank(data)
 
             status = '200 OK'  # HTTP Status
             response = {
-              'responseData': responseData,
+              'responseData': response_data,
               'responseStatus': 200,
               'responseDetails': None,
             }
-    elif path == 'demo':
-        status = '200 OK'  # HTTP Status
-        headers = [('Content-type', 'text/html; charset=utf-8')]  # HTTP Headers
-        start_response(status, headers)
-        return [query_form.format(**environ)]
 
     else:
         # Incorrect URL
@@ -445,16 +351,20 @@ def application(environ, start_response):
 
 def main():
 
+    # lazy imports
+    import argparse
+    import sys
+
+    # parse arguments
     parser = argparse.ArgumentParser()
     parser.add_argument('-s', '--serve', action='store_true', default=False, dest='serve', help='launch web service')
-    parser.add_argument('--host', default=HOST, dest='host', help='host/ip to bind to')
-    parser.add_argument('--port', default=PORT, dest='port', help='port to listen on')
+    parser.add_argument('--host', default=None, dest='host', help='host/ip to bind to')
+    parser.add_argument('--port', default=9008, dest='port', help='port to listen on')
     parser.add_argument('-v', action='count', dest='verbosity', help='increase verbosity (repeat for greater effect)')
     parser.add_argument('-m', dest='model', help='load model from file')
     parser.add_argument('-l', '--langs', dest='langs', help='comma-separated set of target ISO639 language codes (e.g en,de)')
     parser.add_argument('-r', '--remote', action="store_true", default=False, help='auto-detect IP address for remote access')
     parser.add_argument('-b', '--batch', action="store_true", default=False, help='specify a list of files on the command line')
-    parser.add_argument('--demo', action="store_true", default=False, help='launch an in-browser demo application')
     parser.add_argument('-d', '--dist', action='store_true', default=False, help='show full distribution over languages')
     parser.add_argument('-u', '--url', help='langid of URL')
     parser.add_argument('--line', action="store_true", default=False, help='process pipes line-by-line rather than as a document')
@@ -471,16 +381,17 @@ def main():
 
     # unpack a model
     global IDENTIFIER
+
     if options.model:
         try:
             IDENTIFIER = LanguageIdentifier.from_modelpath(options.model, norm_probs=options.normalize)
-            logger.info("Using external model: %s", options.model)
+            LOGGER.info("Using external model: %s", options.model)
         except IOError as e:
-            logger.warning("Failed to load %s: %s" % (options.model, e))
+            LOGGER.warning("Failed to load %s: %s", options.model, e)
 
     if IDENTIFIER is None:
         IDENTIFIER = LanguageIdentifier.from_pickled_model(MODEL_FILE, norm_probs=options.normalize)
-        logger.info("Using internal model")
+        LOGGER.info("Using internal model")
 
     if options.langs:
         langs = options.langs.split(",")
@@ -493,66 +404,50 @@ def main():
         return IDENTIFIER.rank(text) if options.dist else IDENTIFIER.classify(text)
 
     if options.url:
-        import contextlib
         from urllib.request import urlopen
-        with contextlib.closing(urlopen(options.url)) as url:
+        with urlopen(options.url) as url:
             text = url.read()
             output = _process(text)
             print(options.url, len(text), output)
 
-    elif options.serve or options.demo:
+    elif options.serve:
+        import socket
+        from wsgiref.simple_server import make_server
+
         # from http://stackoverflow.com/questions/166506/finding-local-ip-addresses-in-python
         if options.remote and options.host is None:
             # resolve the external ip address
-            import socket
             s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
             s.connect(("google.com", 80))
             hostname = s.getsockname()[0]
         elif options.host is None:
             # resolve the local hostname
-            import socket
             hostname = socket.gethostbyname(socket.gethostname())
         else:
             hostname = options.host
 
-        if options.demo:
-            import webbrowser
-            webbrowser.open('http://{0}:{1}/demo'.format(hostname, options.port))
+        print("Listening on %s:%d" % (hostname, int(options.port)))
+        print("Press Ctrl+C to exit")
+        httpd = make_server(hostname, int(options.port), application)
         try:
-            if FORCE_WSGIREF:
-                raise ImportError
-            # Use fapws3 if available
-            import fapws._evwsgi as evwsgi
-            from fapws import base
-            evwsgi.start(hostname, str(options.port))
-            evwsgi.set_base_module(base)
-            evwsgi.wsgi_cb(('', application))
-            evwsgi.set_debug(0)
-            evwsgi.run()
-        except ImportError:
-            print("Listening on %s:%d" % (hostname, int(options.port)))
-            print("Press Ctrl+C to exit")
-            httpd = make_server(hostname, int(options.port), application)
-            try:
-                httpd.serve_forever()
-            except KeyboardInterrupt:
-                pass
+            httpd.serve_forever()
+        except KeyboardInterrupt:
+            pass
+
     elif options.batch:
         # Start in batch mode - interpret input as paths rather than content
         # to classify.
         import csv
-        import os
-        import sys
-        import multiprocessing as mp
+        from multiprocessing import Pool
 
         def generate_paths():
             for line in sys.stdin:
                 path = line.strip()
-                if path and os.path.isfile(path):
+                if path and Path.is_file(path):
                     yield path
 
         writer = csv.writer(sys.stdout)
-        with mp.Pool() as pool:
+        with Pool() as pool:
             if options.dist:
                 writer.writerow(['path'] + IDENTIFIER.nb_classes)
                 for path, ranking in pool.imap_unordered(rank_path, generate_paths()):
@@ -563,7 +458,6 @@ def main():
                 for path, (lang, conf) in pool.imap_unordered(cl_path, generate_paths()):
                     writer.writerow((path, lang, conf))
     else:
-        import sys
         if sys.stdin.isatty():
             # Interactive mode
             while True:
